@@ -26,10 +26,10 @@ import Text.XML.Light
 
 import Data.List as List
 import qualified Data.Map as Map
-import Data.Maybe
+import Data.Maybe (catMaybes, mapMaybe, maybeToList)
 
-import Control.Monad
-import Control.Monad.Reader
+import Control.Monad (MonadPlus (mzero, mplus), guard, liftM, liftM2)
+import Control.Monad.Reader (ReaderT, runReaderT, ask, lift, withReaderT)
 
 import System.IO (openFile, IOMode (ReadMode), hSetEncoding, utf8, hGetContents)
 
@@ -57,8 +57,8 @@ readFileUTF8 fp = do
 fromStrings :: [String] -> [XHeader]
 fromStrings xs =
    let rs = mapAlt fromString xs
-       Just headers = runReaderT rs headers
-   in headers 
+       headers = concat $ maybeToList $ runReaderT rs headers
+   in headers
 
 -- The 'Parse' monad.  Provides the name of the
 -- current module, and a list of all of the modules.
@@ -120,19 +120,19 @@ findError pname xs =
         Just (XError name code alignment elems) -> Just $ ErrorDetails name code alignment elems
         _ -> error "impossible: fatal error in Data.XCB.FromXML.findError"
     where  f (XError name _ _ _) | name == pname = True
-           f _ = False 
-                                       
+           f _ = False
+
 findEvent :: Name -> [XDecl] -> Maybe EventDetails
-findEvent pname xs = 
+findEvent pname xs =
       case List.find f xs of
         Nothing -> Nothing
-        Just (XEvent name code alignment elems noseq) ->
-            Just $ EventDetails name code alignment elems noseq
+        Just (XEvent name code alignment xge elems noseq) ->
+            Just $ EventDetails name code alignment xge elems noseq
         _ -> error "impossible: fatal error in Data.XCB.FromXML.findEvent"
-   where f (XEvent name _ _ _ _) | name == pname = True
+   where f (XEvent name _ _ _ _ _) | name == pname = True
          f _ = False 
 
-data EventDetails = EventDetails Name Int (Maybe Alignment) [StructElem] (Maybe Bool)
+data EventDetails = EventDetails Name Int (Maybe Alignment) (Maybe Bool) [StructElem] (Maybe Bool)
 data ErrorDetails = ErrorDetails Name Int (Maybe Alignment) [StructElem]
 
 ---
@@ -221,11 +221,12 @@ xevent :: Element -> Parse XDecl
 xevent el = do
   name <- el `attr` "name"
   number <- el `attr` "number" >>= readM
+  let xge = ensureUpper `liftM` (el `attr` "xge") >>= readM
   let noseq = ensureUpper `liftM` (el `attr` "no-sequence-number") >>= readM
   (alignment, xs) <- extractAlignment (elChildren el)
   fields <- mapM structField $ xs
   guard $ not $ null fields
-  return $ XEvent name number alignment fields noseq
+  return $ XEvent name number alignment xge fields noseq
 
 xevcopy :: Element -> Parse XDecl
 xevcopy el = do
@@ -235,12 +236,12 @@ xevcopy el = do
   -- do we have a qualified ref?
   let (mname,evname) = splitRef ref
   details <- lookupEvent mname evname
-  return $ let EventDetails _ _ alignment fields noseq =
+  return $ let EventDetails _ _ alignment xge fields noseq =
                  case details of
                    Nothing ->
                        error $ "Unresolved event: " ++ show mname ++ " " ++ ref
                    Just x -> x  
-           in XEvent name number alignment fields noseq
+           in XEvent name number alignment xge fields noseq
 
 -- we need to do string processing to distinguish qualified from
 -- unqualified types.
@@ -262,10 +263,10 @@ split :: Char -> String -> (String, String)
 split c = go
     where go [] = ([],[])
           go (x:xs) | x == c = ([],xs)
-                    | otherwise = 
+                    | otherwise =
                         let (lefts, rights) = go xs
                         in (x:lefts,rights)
-                 
+
 
 xerror :: Element -> Parse XDecl
 xerror el = do
@@ -348,8 +349,10 @@ structField el
         return $ SField name typ enum mask
 
     | el `named` "pad" = do
-        bytes <- el `attr` "bytes" >>= readM
-        return $ Pad bytes
+        let bytes = liftM (Pad PadBytes) $ el `attr` "bytes" >>= readM
+        let align = liftM (Pad PadAlignment) $ el `attr` "align" >>= readM
+
+        return $ head $ catMaybes $ [bytes, align]
 
     | el `named` "list" = do
         typ <- liftM mkType $ el `attr` "type"
@@ -508,4 +511,3 @@ children :: MonadPlus m => Element -> String -> m [Element]
 -- adapted from Network.CGI.Protocol
 readM :: (MonadPlus m, Read a) => String -> m a
 readM = liftM fst . listToM . reads
-
