@@ -46,6 +46,7 @@ module Data.XCB.Types
     , MaskPadding
     , Alignment ( .. )
     , AllowedEvent ( .. )
+    , PadType ( .. )
     ) where
 
 import Data.Map
@@ -82,7 +83,7 @@ type XEnumElem = EnumElem Type
 data GenXDecl typ
     = XStruct  Name (Maybe Alignment) [GenStructElem typ]
     | XTypeDef Name typ
-    | XEvent Name Int (Maybe Alignment) [GenStructElem typ] (Maybe Bool)  -- ^ The boolean indicates if the event includes a sequence number.
+    | XEvent Name Int (Maybe Alignment) (Maybe Bool) [GenStructElem typ] (Maybe Bool)  -- ^ bools: #1 if xge is true; #2  if the event includes a sequence number.
     | XRequest Name Int (Maybe Alignment) [GenStructElem typ] (Maybe (GenXReply typ))
     | XidType  Name
     | XidUnion  Name [GenXidUnionElem typ]
@@ -93,8 +94,13 @@ data GenXDecl typ
     | XEventStruct Name [AllowedEvent]
  deriving (Show, Functor)
 
+data PadType
+    = PadBytes
+    | PadAlignment
+ deriving (Show)
+
 data GenStructElem typ
-    = Pad Int
+    = Pad PadType Int
     | List Name typ (Maybe (Expression typ)) (Maybe (EnumVals typ))
     | SField Name typ (Maybe (EnumVals typ)) (Maybe (MaskVals typ))
     | ExprField Name typ (Expression typ)
